/*
This software is released under the DWYWWIIINRC (Do Whatever You Want With It, It Is Not Rocket Science) License.
This software is used for monitoring an eletric meter with a 1 Wh LED pulse signal. 
Read more at 
http://jespereklund.blogspot.com/2011/10/elmaler-overvagning-med-arduino.html
The software uses a Sparkfun Graphic LCD 128x64 display:
http://www.sparkfun.com/products/710
*/

#include <NewSoftSerial.h>
#include <stdlib.h>

//pins
int LCDPin = 8;
int LDRPin = 3;

//tabs for text align
int tab1 = 1;
int tab2 = 23;
int tab3 = 64; 
int tab4 = 85; 

//misc vars for statistics
int count = 0;
int error = 0;
float wCurrent;
float wAvg;
unsigned long timeNow;
unsigned long timeOld = 0.0;
unsigned long timeStart = 0.0;
float wMin = 10000.0;
float wMax = 0.0;
float sec = 0.0; 
float timeInterval = 0.0;
boolean doFirst = true;


//attach display
NewSoftSerial lcdSerialPort(3, LCDPin);

void setup() {
  //init display
  setBaudRate(9600);
  clearScreen();
  setBackgroundBrightness(50);

  //print all stationary on display
  setLine(0);
  setX(tab1);
  print("Cur");

  setLine(1);
  setX(tab1);
  print("Min");

  setLine(2);
  setX(tab1);
  print("Max");

  setLine(3);
  setX(tab1);
  print("Avg");

  setLine(4);
  setX(tab1);
  print("Wh");

  setLine(5);
  setX(tab1);
  print("Sek");

  setLine(0);
  setX(tab3);
  print("Err");

  //attach interrupt for LDR trigger
  attachInterrupt(1, doCalc, FALLING);  
}

void loop()
{
  //update display every 5 second
  updateDisplay();
  delay(5000);
}

//Updates all non-stationary text on display
void updateDisplay() {
  //Current
  setLine(0);
  setX(tab2);
  printF(wCurrent,0);

  //Min
  setLine(1);
  setX(tab2);
  printF(wMin,0);

  //Max
  setLine(2);
  setX(tab2);
  printF(wMax,0);

  //Avg
  setLine(3);
  setX(tab2);
  printF(wAvg,0);

  //Total Wh
  setLine(4);
  setX(tab2);
  printF(count,0);

  //Duration
  setLine(5);
  setX(tab2);
  printF(sec,0);

  //Errors
  setLine(0);
  setX(tab4);
  printF(error,0);
}

//calculates watt and all statistics
void doCalc() {
  //avoid false min and max by skipping those calcs in first round.
  if ( doFirst ) {
    timeStart = millis();
    timeOld = millis(); 
    doFirst = false;
    //skibs the rest the first time
    return;
  }
  
  //Calculate statistics
  timeNow = millis();
  timeInterval = (timeNow - timeOld) / 1000.0;
  //detect error prel pulse
  if ( timeInterval > 0.1 ) {
    timeOld = timeNow;  
    count++;
    sec = ( timeNow - timeStart ) / 1000.0;
    wAvg = 3600.0 * count / sec;
    wCurrent = 3600.00 / timeInterval;
    if ( wCurrent > wMax ) { wMax = wCurrent; }
    if ( wCurrent < wMin ) { wMin = wCurrent; }
  } else {
    error++; 
  }
}
  
//print nice numbers  
void printF(float val, int format) {
  char b[25];
  print(dtostrf(val,5,0,b));
}

//convert from sample int value to float volts
float a2volt(int val) {
  //VCC mlt til 5.15 Volt
  return 5.15 * val / 1024.0;
}

//move display cursor to specified line
void setLine(int line) {
  setY(62-11*line); 
}

//print test on display
void print(char *data){
  lcdSerialPort.print(data);
}



/*******************************************************
The following display functions is obtained from 
http://forum.sparkfun.com/viewtopic.php?f=8&t=20829
*/

void clearScreen(){
  lcdSerialPort.print(0x7C,BYTE);
  lcdSerialPort.print(0x00,BYTE);
}

void setBaudRate(long value){
  // Get the internal reference for this baud rate
  char *ref = " ";
  if(value == 4800)
    ref = "1";
  else if(value == 9600)
    ref = "2";
  else if(value == 19200)
    ref = "3";
  else if(value == 38400)
    ref = "4";
  else if(value == 57600)
    ref = "5";
  else if(value == 115200)
    ref = "6";
  else
    return;

  // Since it often rolls back to 115200, try setting it via 115200 first
  lcdSerialPort.begin(115200);
  lcdSerialPort.print(0x7C,BYTE);
  lcdSerialPort.print(0x07,BYTE);
  lcdSerialPort.print(ref);

  // Now change the serial port to the desired rate, and set it again.
  lcdSerialPort.begin(value);
  lcdSerialPort.print(0x7C,BYTE);
  lcdSerialPort.print(0x07,BYTE);
  lcdSerialPort.print(ref);
}

void setX(byte value){
  lcdSerialPort.print(0x7C,BYTE);
  lcdSerialPort.print(0x18,BYTE);
  lcdSerialPort.print(value,BYTE);
}

void setBackgroundBrightness(byte value){
  lcdSerialPort.print(0x7C,BYTE);
  lcdSerialPort.print(0x02,BYTE);
  lcdSerialPort.print(value,BYTE);
}

void setY(byte value){
  lcdSerialPort.print(0x7C,BYTE);
  lcdSerialPort.print(0x19,BYTE);
  lcdSerialPort.print(value,BYTE);
}




